### Multinomial Logit model estimation

#パッケージ読み込み
library(maxLik)
library(data.table)
library(tidyverse)
library(abind)
library(stringr)
library(lubridate)
library(scales)

#作業ディレクトリ
#setwd("C:/Users/h.shiroma/Desktop/モデル推定")
setwd("E:/Dropbox (fukudalab-tokyotech)/夏の学校2018")

#データ読み込み
Data <- read.csv("目的地選択モデル用データセット4_残業ダミーつき.csv",header=TRUE,
                 fileEncoding="CP932",stringsAsFactors=FALSE)

Data %>%
  filter(Modecode == "200"|Modecode == "100"|Modecode == "240") %>%
  mutate(Purpusecode = dplyr::if_else(Purpusecode != as.integer(400),Purpusecode,as.integer(100)))-> Data

# サンプル数
hh <- nrow(Data)

## パラメータ初期値設定
bmode0 <- numeric(4)

##### MNL対数尤度関数の定義 #####
#x <- c(1,1,1,1)
fr <- function(x) {
  ### 推定パラメータの定義
  ## 定数項
  ASC_car <- x[1]
  ASC_bus <- x[2]
  
  ## 旅行コスト
  bcost <- x[3]
  #bcost2 <- x[4]
  ## 旅行時間
  btime <- x[4]
  
  ## 対数尤度のオブジェクト作成
  LL = 0
  
  ## 効用関数の定義
  
  #ロジット選択確率の分子
  car      <- Data$AMGcar_LOS_A  *exp(bcost*Data$TC_car_LOS_A/100    + btime*Data$TT_car_LOS_A/100   + ASC_car*matrix(1,nrow =hh,ncol=1))
  bus      <- Data$AMGbus_LOS_A  *exp(bcost*Data$TC_bus_LOS_A/100    + btime*Data$TT_bus_LOS_A/100   + ASC_bus*matrix(1,nrow =hh,ncol=1))
  train    <- Data$AMGtrain_LOS_A*exp(bcost*Data$TC_train_LOS_A/100  + btime*Data$TT_train_LOS_A/100 + matrix(0,nrow =hh,ncol=1))

  #ロジット選択確率の分母
  deno <- (car + train + bus)
  
  ## ロジット型選択確率の計算
  Ptrain <- Data$AMGcar_LOS_A*(train / deno)
  Pbus   <- Data$AMGbus_LOS_A *(bus   / deno)
  Pcar   <- Data$AMGtrain_LOS_A *(car   / deno)
 
  ## 選択確率が0になったときの対応
  Ptrain <- (Ptrain!=0)*Ptrain + (Ptrain==0)
  Pbus   <- (Pbus!=0)*Pbus     + (Pbus==0)
  Pcar   <- (Pcar  !=0)*Pcar   + (Pcar  ==0)

  ## 選択結果
  Ctrain   <- Data$Modecode =="200"
  Cbus     <- Data$Modecode =="240"
  Ccar     <- Data$Modecode =="100"

  ## 対数尤度の計算(同時確率の計算)
  LL <- colSums(Ctrain*log(Ptrain) + Cbus*log(Pbus) + Ccar*log(Pcar))
}

##### 対数尤度関数frの最大化#####

##パラメータ値の最適化
resmode <- maxLik(fr,start=bmode0,method="BFGS",control=list(printLevel=2))

#初期尤度
print(fr(bmode0))
## パラメータ推定値
##　ヘッセ行列
bmode <- resmode$estimate
hhh <- resmode$hessian
print(hhh)
##逆行列計算
print(solve(hhh))
## t値
tval <- bmode/sqrt(-diag(solve(hhh)))

## 初期尤度
L0 <- fr(bmode0)
## 最終尤度
LL <- resmode$maximum
summary(resmode)

##### 結果の出力 #####
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(bmode)))/L0)
##パラメータ推定値 
print(bmode)
## t値 
print(tval)

################################################################
#時間帯選択モデル
################################################################

#モード選択ログサム計算
time <- c("A") #i <- "A"
logsum_mode <- Data$Trip_ID
for (i in time){
  Data %>%
    select(contains("_LOS_")) %>%
    select(ends_with(i)) -> DataLOS
  
  DataLOS %>%
    rename_(.dots = setNames(names(DataLOS),str_sub(colnames(DataLOS),end = -2))) %>%
    ## 選択肢となる時間帯別にモード選択のexp(効用)を計算
    mutate(exV_carM = AMGcar_LOS_M*exp(bmode[1]+bmode[3]*TC_car_LOS_M/100+bmode[4]*TT_car_LOS_M/100),
           exV_busM = AMGbus_LOS_M*exp(bmode[2]+bmode[3]*TC_bus_LOS_M/100+bmode[4]*TT_bus_LOS_M/100),
           exV_trainM = AMGtrain_LOS_M*exp(bmode[3]*TC_train_LOS_M/100+bmode[4]*TT_train_LOS_M/100),
           exV_carN = AMGcar_LOS_N*exp(bmode[1]+bmode[3]*TC_car_LOS_N/100+bmode[4]*TT_car_LOS_N/100),
           exV_busN = AMGbus_LOS_N*exp(bmode[2]+bmode[3]*TC_bus_LOS_N/100+bmode[4]*TT_bus_LOS_N/100),
           exV_trainN = AMGtrain_LOS_N*exp(bmode[3]*TC_train_LOS_N/100+bmode[4]*TT_train_LOS_N/100),
           exV_carE = AMGcar_LOS_E*exp(bmode[1]+bmode[3]*TC_car_LOS_E/100+bmode[4]*TT_car_LOS_E/100),
           exV_busE = AMGbus_LOS_E*exp(bmode[2]+bmode[3]*TC_bus_LOS_E/100+bmode[4]*TT_bus_LOS_E/100),
           exV_trainE = AMGtrain_LOS_E*exp(bmode[3]*TC_train_LOS_E/100+bmode[4]*TT_train_LOS_E/100) ) %>%
    ## 選択肢となる目的地別にモード選択の合成効用を計算(ログサム)←アクセシビリティ指標
    mutate(logsum_modeM = log(exV_carM + exV_busM + exV_trainM),
           logsum_modeN = log(exV_carN + exV_busN + exV_trainN),
           logsum_modeE = log(exV_carE + exV_busE + exV_trainE)) %>%
    select(logsum_modeM,logsum_modeN,logsum_modeE) -> DataLOS
  DataLOS %>%
    rename_(.dots = setNames(names(DataLOS),paste(colnames(DataLOS),i,sep = ""))) -> DataLOS
  
  logsum_mode <- cbind(logsum_mode,DataLOS)
}

## パラメータの初期値設定
btime0 <- numeric(7)
btime0[3] <- 0.5
#Data %>%filter(TripID_day == 1) -> Data1
## 対数尤度関数の定義
fr <- function(x) { #x <- c(0,0,0.5,0,0,0,0,0,0)
  ### 推定パラメータ###
  ## 定数項
  ASC_M <- x[1]
  ASC_N <- x[2]
  
  #スケールパラメータ
  pa <- x[3] 
  
  ## トリップ目的
  work_M <- x[4]
  work_N <- x[5]
  home_M <- x[6]
  home_N <- x[7]
  #private_M <- x[8]
  #private_N <- x[9]
  
 
  ## 休日ダミー
  #holiday_M <- x[10]
  #holiday_N <- x[11]
  
  
  ## 対数尤度のオブジェクト作成
  LL = 0
  
  ## 時間帯選択確率の計算(分子)
  Morning      <- Data$Mpeak*exp(pa*(ASC_M*matrix(1,nrow =hh,ncol=1) +logsum_mode$logsum_modeMA +
                                       work_M*(Data$Purpusecode == 100) + 
                                       home_M*(Data$Purpusecode == 200)))
  Noon      <- Data$Noffpeak*exp(pa*(ASC_N*matrix(1,nrow =hh,ncol=1) + logsum_mode$logsum_modeNA +
                                       work_N*(Data$Purpusecode == 100) + 
                                       home_N*(Data$Purpusecode == 200)))
  Evening      <- Data$Apeak*exp(pa*(matrix(0,nrow =hh,ncol=1) + logsum_mode$logsum_modeEA ))
  
  ###  時間帯選択確率の計算(分母)
  ## calculate the Inclusive Value (the denominator of the choice probabilities equation)
  deno <- (Morning + Noon + Evening)
  
  ## 選択確率の計算
  PM <- Data$Mpeak*(Morning / deno)
  PN <- Data$Noffpeak*(Noon / deno)
  PE <- Data$Apeak*(Evening / deno)
  
  ## 選択確率が0になってしまった場合に起こる問題回避
  PM <- (PM!=0)*PM + (PM==0)
  PN <- (PN!=0)*PN + (PN==0)
  PE <- (PE!=0)*PE + (PE==0)
  
  #選択結果
  CM <- Data$timezone == 1
  CN <- Data$timezone == 2
  CE <- Data$timezone == 3
  
  ## 対数尤度関数
  LL <- colSums(CM*log(PM)+CN*log(PN)+CE*log(PE))
}
##### 対数尤度関数の最大化#####
restime <- maxLik(fr,start=btime0,method="BFGS",control=list(printLevel=2))

#目的地選択モデル推定 パラメータ
btime <- restime$estimate
#ヘッセ行列
hhh <- restime$hessian
#逆行列計算
print(solve(hhh))
## t値
tval <- btime/sqrt(-diag(solve(hhh)))
## 初期尤度
L0 <- fr(btime0)
## 最終尤度
LL <- restime$maximum

##### 結果の出力#####
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(btime)))/L0)
##パラメータ推定値 
print(btime)
## t値 
print(tval)



#############################################################################
#############################################################################
##目的地選択
#############################################################################
##モード選択ログサム
time <- c("A","B","C","D","E") #i <- "A"
logsum_mode <- Data$Trip_ID
for (i in time){
  Data %>%
    select(contains("_LOS_")) %>%
    select(ends_with(i)) -> DataLOS
  
  DataLOS %>%
    rename_(.dots = setNames(names(DataLOS),str_sub(colnames(DataLOS),end = -2))) %>%
    ## 選択肢となる目的地毎にモード選択のexp(効用)を計算
    mutate(exV_carM = AMGcar_LOS_M*exp(bmode[1]+bmode[3]*TC_car_LOS_M/100+bmode[4]*TT_car_LOS_M/100),
           exV_busM = AMGbus_LOS_M*exp(bmode[2]+bmode[3]*TC_bus_LOS_M/100+bmode[4]*TT_bus_LOS_M/100),
           exV_trainM = AMGtrain_LOS_M*exp(bmode[3]*TC_train_LOS_M/100+bmode[4]*TT_train_LOS_M/100),
           exV_carN = AMGcar_LOS_N*exp(bmode[1]+bmode[3]*TC_car_LOS_N/100+bmode[4]*TT_car_LOS_N/100),
           exV_busN = AMGbus_LOS_N*exp(bmode[2]+bmode[3]*TC_bus_LOS_N/100+bmode[4]*TT_bus_LOS_N/100),
           exV_trainN = AMGtrain_LOS_N*exp(bmode[3]*TC_train_LOS_N/100+bmode[4]*TT_train_LOS_N/100),
           exV_carE = AMGcar_LOS_E*exp(bmode[1]+bmode[3]*TC_car_LOS_E/100+bmode[4]*TT_car_LOS_E/100),
           exV_busE = AMGbus_LOS_E*exp(bmode[2]+bmode[3]*TC_bus_LOS_E/100+bmode[4]*TT_bus_LOS_E/100),
           exV_trainE = AMGtrain_LOS_E*exp(bmode[3]*TC_train_LOS_E/100+bmode[4]*TT_train_LOS_E/100) ) %>%
    ## 選択肢となる目的地別にモード選択の合成効用を計算(ログサム)←アクセシビリティ指標
    mutate(logsum_modeM = log(exV_carM + exV_busM + exV_trainM),
           logsum_modeN = log(exV_carN + exV_busN + exV_trainN),
           logsum_modeE = log(exV_carE + exV_busE + exV_trainE)) %>%
    select(logsum_modeM,logsum_modeN,logsum_modeE) -> DataLOS
  DataLOS %>%
    rename_(.dots = setNames(names(DataLOS),paste(colnames(DataLOS),i,sep = ""))) -> DataLOS
  
  logsum_mode <- cbind(logsum_mode,DataLOS)
}

##時間帯選択ログサム
Data %>%
  mutate(#M
         exVmpeakA = Mpeak*exp(btime[1] + logsum_mode$logsum_modeMA +
                                          btime[4]*(Purpusecode == 100) + 
                                          btime[6]*(Purpusecode == 200)),
         exVmpeakB = Mpeak*exp(btime[1] + logsum_mode$logsum_modeMB +
                                           btime[4]*(Purpusecode == 100) + 
                                           btime[6]*(Purpusecode == 200)),
         exVmpeakC = Mpeak*exp(btime[1] + logsum_mode$logsum_modeMC +
                                           btime[4]*(Purpusecode == 100) + 
                                           btime[6]*(Purpusecode == 200)),
         exVmpeakD = Mpeak*exp(btime[1] + logsum_mode$logsum_modeME +
                                           btime[4]*(Purpusecode == 100) + 
                                           btime[6]*(Purpusecode == 200)),
         exVmpeakE = Mpeak*exp(btime[1] + logsum_mode$logsum_modeMD +
                                           btime[4]*(Purpusecode == 100) + 
                                           btime[6]*(Purpusecode == 200)),
         #N
         exVnoffA = Noffpeak*exp(btime[2] + logsum_mode$logsum_modeNA +
                                         btime[5]*(Purpusecode == 100) + 
                                         btime[7]*(Purpusecode == 200)),
         exVnoffB = Noffpeak*exp(btime[2] + logsum_mode$logsum_modeNB +
                                            btime[5]*(Purpusecode == 100) + 
                                            btime[7]*(Purpusecode == 200)),
         exVnoffC = Noffpeak*exp(btime[2] + logsum_mode$logsum_modeNC +
                                            btime[5]*(Purpusecode == 100) + 
                                            btime[7]*(Purpusecode == 200)),
         exVnoffD = Noffpeak*exp(btime[2] + logsum_mode$logsum_modeND +
                                            btime[5]*(Purpusecode == 100) + 
                                            btime[7]*(Purpusecode == 200)),
         exVnoffE = Noffpeak*exp(btime[2] + logsum_mode$logsum_modeNE +
                                            btime[5]*(Purpusecode == 100) + 
                                            btime[7]*(Purpusecode == 200)),
         #E
         exVapeakA = Apeak*exp(logsum_mode$logsum_modeEA ),
         exVapeakB = Apeak*exp(logsum_mode$logsum_modeEB ),
         exVapeakC = Apeak*exp(logsum_mode$logsum_modeEC ),
         exVapeakD = Apeak*exp(logsum_mode$logsum_modeED ),
         exVapeakE = Apeak*exp(logsum_mode$logsum_modeEE )) %>%
  mutate(logsum_timeA = log(exVmpeakA + exVnoffA + exVapeakA),
         logsum_timeB = log(exVmpeakB + exVnoffB + exVapeakB),
         logsum_timeC = log(exVmpeakC + exVnoffC + exVapeakC),
         logsum_timeD = log(exVmpeakD + exVnoffD + exVapeakD),
         logsum_timeE = log(exVmpeakE + exVnoffE + exVapeakE)) -> Data

## パラメータの初期値設定
bdest0 <- c(17.222967,19.173474,6.417038,0.1) #MNL推定結果 17.222967 19.173474  6.417038
bdest0[4] <- 0.1
bdest0[1] <- bdest00[1]
bdest0[2] <- bdest00[2]
bdest0[3] <- bdest00[3]
##### 目的地選択モデル対数尤度関数 #####
fr <- function(x) { #x <- c(0,0,0.5)
  ### 推定パラメータ###
  
  ## 目的地属性
  #業務用地面積
  bBUbusi <- x[1]
  #商業用地面積
  bBUcomm <- x[2]
  bBUhome <- x[3]
  
  ##スケールパラメータ
  pa <- x[4]   
  
  ## 対数尤度のオブジェクト作成
  LL = 0
  
  ## 目的地選択確率の計算(分子)
  DA      <- exp(pa*((Data$Purpusecode == 100)*bBUbusi*(Data$LUbusinessA/100)  +
                       (Data$Purpusecode == 777)*bBUcomm*((Data$LUcommerceA)/100)  + 
                       (Data$Purpusecode == 200)*bBUhome*(Data$LUhouseA/100)  +Data$logsum_timeA ))
                       #(Data$Purpusecode == 200)*bBUhome*(Data$LUhouseA/100)))
  DB      <- exp(pa*((Data$Purpusecode == 100)*bBUbusi*(Data$LUbusinessB/100)  + 
                       (Data$Purpusecode == 777)*bBUcomm*(Data$LUcommerceB/100)   + 
                       (Data$Purpusecode == 200)*bBUhome*(Data$LUhouseB/100)  +   Data$logsum_timeB ))
                       #(Data$Purpusecode == 200)*bBUhome*(Data$LUhouseB/100) ))
  
  DC      <- exp(pa*((Data$Purpusecode == 100)*bBUbusi*(Data$LUbusinessC/100)  + 
                       (Data$Purpusecode == 777)*bBUcomm*(Data$LUcommerceC/100)   + 
                       (Data$Purpusecode == 200)*bBUhome*(Data$LUhouseC/100)  + Data$logsum_timeC ))
                       #(Data$Purpusecode == 200)*bBUhome*(Data$LUhouseC/100)))
  DD      <- exp(pa*((Data$Purpusecode == 100)*bBUbusi*(Data$LUbusinessD/100)  + 
                       (Data$Purpusecode == 777)*bBUcomm*(Data$LUcommerceD/100)   + 
                       (Data$Purpusecode == 200)*bBUhome*(Data$LUhouseD/100) + Data$logsum_timeD ))
                       #(Data$Purpusecode == 200)*bBUhome*(Data$LUhouseD/100)))
  DE      <- exp(pa*((Data$Purpusecode == 100)*bBUbusi*(Data$LUbusinessE/100)  + 
                       (Data$Purpusecode == 777)*bBUcomm*(Data$LUcommerceE/100)   + 
                       (Data$Purpusecode == 200)*bBUhome*(Data$LUhouseE/100)  + Data$logsum_timeE ))
                       #(Data$Purpusecode == 200)*bBUhome*(Data$LUhouseE/100)))
  
  
  ###  目的地選択確率の計算(分母)
  ## calculate the Inclusive Value (the denominator of the choice probabilities equation)
  deno <- (DA + DB + DC + DD + DE)
  
  ## 選択確率の計算(目的地Aのみ)
  PA <- (DA / deno)
  
  ## 選択確率が0になってしまった場合に起こる問題回避
  PA <- (PA!=0)*PA + (PA==0)
  
  #選択結果(目的地A)
  CA <- matrix(1,nrow =hh,ncol=1)
  
  ## 対数尤度関数
  LL <- colSums(CA*log(PA))
  
}
##### 対数尤度関数の最大化#####
resdest <- maxLik(fr,start=bdest0,method="SANN",control=list(printLevel=2))

summary(resdest)
#目的地選択モデル推定 パラメータ
#bdest00 <- resdest$estimate
bdest0 <- resdest$estimate
print(fr(numeric(4)))
#ヘッセ行列
hhh <- resdest$hessian
#逆行列計算
print(solve(hhh))
## t値
tval <- bdest/sqrt(-diag(solve(hhh)))
## 初期尤度 m <- c(0,0,0,0)
L0 <- fr(m)
## 最終尤度
LL <- resdest$maximum

##### 結果の出力#####
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(bdest)))/L0)
##パラメータ推定値 
print(bdest)
## t値 
print(tval)

bdest <- bdest0
#######################################################################
#パターン選択モデル
####################################################################

#時間帯選択ログサム計算
Data %>%
  mutate(exVA = exp((Purpusecode == 100)*bdest[1]*(LUbusinessA/100)  +
                          (Purpusecode == 777)*bdest[2]*(LUcommerceA/100)  + 
                          (Purpusecode == 200)*bdest[3]*(LUhouseA/100)  +Data$logsum_timeA ),
         exVB = exp((Purpusecode == 100)*bdest[1]*(LUbusinessB/100)  +
                             (Purpusecode == 777)*bdest[2]*(LUcommerceB/100)  + 
                             (Purpusecode == 200)*bdest[3]*(LUhouseB/100)  +Data$logsum_timeB ),
         exVC = exp((Purpusecode == 100)*bdest[1]*(LUbusinessC/100)  +
                      (Purpusecode == 777)*bdest[2]*(LUcommerceC/100)  + 
                      (Purpusecode == 200)*bdest[3]*(LUhouseC/100)  +Data$logsum_timeC ),
         exVD = exp((Purpusecode == 100)*bdest[1]*(LUbusinessD/100)  +
                      (Purpusecode == 777)*bdest[2]*(LUcommerceD/100)  + 
                      (Purpusecode == 200)*bdest[3]*(LUhouseD/100)  +Data$logsum_timeD ),
         exVE = exp((Purpusecode == 100)*bdest[1]*(LUbusinessE/100)  +
                      (Purpusecode == 777)*bdest[2]*(LUcommerceE/100)  + 
                      (Purpusecode == 200)*bdest[3]*(LUhouseE/100)  +Data$logsum_timeE )) %>%
  mutate(logsum_dest = log(exVA + exVB + exVC + exVD + exVE )) %>%
  #全アクティビティでの期待効用合計
  group_by(Monitor_ID,Obdate,Acvpattern) %>%
  mutate(logsum_pattern = sum(logsum_dest)) %>%
  ungroup() -> Data

Data %>%
  select(Monitor_ID,Obdate,Acvpattern,Age,Sexcode,logsum_pattern) %>%
  distinct() -> Data

Data %>%
  mutate(Wday = wday(Obdate,label=TRUE)) %>%
  mutate(Day_Dummy = dplyr::case_when(
    .$Wday == "Mon" ~ 1,
    .$Wday == "Tues" ~ 1,
    .$Wday == "Wed" ~ 1,
    .$Wday == "Thurs" ~ 1,
    .$Wday == "Fri" ~ 1,
    .$Wday == "Sat" ~ 0,
    .$Wday == "Sun" ~ 0
  )) -> Data
## パラメータの初期値設定
hh <- nrow(Data)
bpattern0 <- numeric(5)
bpattern0[5] <- 0.5

## 対数尤度関数の定義
fr <- function(x) { #x <- c(0,0,0.5,0,0)
  ### 推定パラメータ###
  ## 定数項
  ASC_p1 <- x[1]
  ASC_p2 <- x[2]
  
  #スケールパラメータ
  pa <- x[3] 
  
  #説明変数パラメータ
  holiday_p2 <- x[4]
  holiday_p3 <- x[5]
  
  ## 対数尤度のオブジェクト作成
  LL = 0
  
  ## 時間帯選択確率の計算(分子)
  p1      <- exp(pa*(ASC_p1*matrix(1,nrow =hh,ncol=1)　+ Data$logsum_pattern))
  p2      <- exp(pa*(ASC_p2*matrix(1,nrow =hh,ncol=1) + holiday_p2*(Data$Day_Dummy == 0) + Data$logsum_pattern))
  p3      <- exp(pa*(holiday_p3*(Data$Day_Dummy == 0) + Data$logsum_pattern))
  
  ###  時間帯選択確率の計算(分母)
  ## calculate the Inclusive Value (the denominator of the choice probabilities equation)
  deno <- (p1 + p2 + p3)
  
  ## 選択確率の計算
  Pp1 <- (p1 / deno)
  Pp2 <- (p2 / deno)
  Pp3 <- (p3 / deno)
  
  ## 選択確率が0になってしまった場合に起こる問題回避
  Pp1 <- (Pp1!=0)*Pp1 + (Pp1==0)
  Pp2 <- (Pp2!=0)*Pp2 + (Pp2==0)
  Pp3 <- (Pp3!=0)*Pp3 + (Pp3==0)
  
  #選択結果
  Cp1 <- Data$Acvpattern == "出勤→帰宅" 
  Cp2 <- Data$Acvpattern == "出勤→私事→帰宅"
  Cp3 <- Data$Acvpattern == "私事→帰宅"
  
  ## 対数尤度関数
  LL <- colSums(Cp1*log(Pp1)+Cp2*log(Pp2)+Cp3*log(Pp3))
}
##### 対数尤度関数の最大化#####
respattern <- maxLik(fr,start=bpattern0,method="SANN",control=list(printLevel=2))
summary(respattern)
#目的地選択モデル推定 パラメータ
bpattern <- respattern$estimate
#ヘッセ行列
hhh <- respattern$hessian
#逆行列計算
print(solve(hhh))
## t値
tval <- bpattern/sqrt(-diag(solve(hhh)))
## 初期尤度
L0 <- fr(bpattern0)
## 最終尤度
LL <- respattern$maximum

##### 結果の出力#####
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(bpattern)))/L0)
##パラメータ推定値 
print(bpattern)
## t値 
print(tval)

#
bpattern[3]/btime[3]

summary(resmode)
summary(resdest)
summary(restime)
summary(respattern)


## 初期尤度 m <- c(0,0,0,1) m<-c(0,0,0,0)
L0mode <- fr(bmode0)
## 最終尤度
LLmode <- resdest$maximum

##### 結果の出力#####
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(bdest)))/L0)











